@@ -1,20 +1,28 @@
 from argparse import ArgumentParser
 from pathlib import Path
+from typing import List, Optional
 
 parser = ArgumentParser(
     allow_abbrev=False, description='Script dependency visualiser.'
 )
 
 group_mode = parser.add_argument_group('Source and output')
-group_mode.add_argument('names', nargs='?', default=None, help=(
-    'a colon-delimited sequence of file names source:graph:render. '
-    'Hyphens (-) DISABLE input/output (e.g. do not write a graph: s:-:r). '
-    'If source is disabled, graph is used as the only input instead. '
-    'Disabling two or more components will return with an error code 1. '
-    'Empty graph and render names (e.g. s::) are INFERRED from source and graph, '
-    'respectively. Source must never be empty. '
-    'Several SHORTCUTS are possible by omitting components: '
-    's = s:-:, s:r = s:-:r, -:g = -:g:, s:- = s::-'
+group_mode.add_argument('sources', nargs='*', help=(
+    'source file names. If sources was disabled by providing no names, '
+    '--graph is used as direct input for rendering. Disabling two or more of '
+    'SOURCES, --graph and --output will return with an error code 1.'
+))
+group_mode.add_argument('--graph', '-g', nargs='?', default='-', const=None, help=(
+    'write or read intermediate graph file. Graph output is disabled by default. '
+    'If a value is specified, it is used as the file name. '
+    'If no value is provided, the name is inferred from combining SOURCES. '
+    'See SOURCES for more information.'
+))
+group_mode.add_argument('--output', '-o', help=(
+    'render file name. By default the name is inferred from --graph. '
+    'If the name is a single hyphen, render output is disabled '
+    'and a graph is written to --graph. See SOURCES for more information. '
+    'NOTE: --format is appended to the name'
 ))
 group_mode.add_argument('--format', '-f', default='svg', help=(
     'render file format, appended to all render file names (default: %(default)s) '
@@ -28,7 +36,19 @@ group_mode.add_argument(
 
 group_parse = parser.add_argument_group('Parsing options')
 group_parse.add_argument(
-    '--skip-imports', '-si', action='store_true', help='do not visualise imports'
+    '--imports', '-i', default='interface', help=(
+        'duplicate or resolve import nodes. '
+        'Valid values are duplicate, interface and resolve (default: %(default)s). '
+        'Duplicating produces a node for each import in the importing source. '
+        'Resolving imports links edges directly to the original definitions instead. '
+        '"interface" leaves import nodes that reference definitions directly below '
+        'the import in the module hierarchy and resolves others.'
+    )
+)
+group_parse.add_argument(
+    '--skip-external', '-se', action='store_true', help=(
+        'do not visualise imports to external modules'
+    )
 )
 group_parse.add_argument(
     '--exclude', '-e', action='append', help='exclude nodes in the source'
@@ -44,6 +64,13 @@ group_parse.add_argument(
         'when --collapse-waypoints is set'
     )
 )
+group_parse.add_argument(
+    '--root', '-r', default=None, help=(
+        'only show root and its children in the graph '
+        'NOTE: does not affect graph coloring'
+    )
+)
+
 group_graph = parser.add_argument_group('Graph appearance')
 group_graph.add_argument(
     '--stagger', type=int, default=2, help='max Graphviz unflatten stagger'
@@ -93,43 +120,39 @@ class ArgumentError(RuntimeError):
     """Invalid command line arguments given."""
 
 
-def parse_names(names):
+def make_absolute(path: Path) -> Path:
+    """Make absolute path out of a potentially relative one."""
+    return path if path.is_absolute() else Path.cwd() / path
+
+
+def infer_graph_from_sources(sources: List[str]) -> Path:
+    """Infer graph name from sources."""
+    parts = [make_absolute(Path(s)).stem for s in sources]
+    return Path('-'.join(parts)).with_suffix('.gv')
+
+
+def parse_names(sources: List[str], graph: Optional[str], output: Optional[str]):
     """Parse file names from arguments."""
-    split = names.split(':')
+    if not sources and (not graph or graph == '-'):
+        raise ArgumentError('Pyfactor: no input specified!')
+    if graph == '-' and output == '-':
+        raise ArgumentError('Pyfactor: all output disabled!')
+    if not sources and output == '-':
+        raise ArgumentError('Pyfactor: only graph name specified!')
 
-    if sum(s == '-' for s in split) > 1:
-        raise ArgumentError(f'Pyfactor: too many disabled names in `{names}`!')
-    if split[0] == '' or (split[0] == '-' and split[1] == ''):
-        raise ArgumentError(f'Pyfactor: cannot infer names from `{names}`!')
-    if len(split) > 3:
-        raise ArgumentError(f'Pyfactor: too many names in `{names}`!')
+    if not sources:
+        o = output or str(Path(graph).with_suffix(''))
+        return None, graph, o
 
-    # Expand to three components
-    if len(split) == 1:
-        split += ['-', '']
-    if len(split) == 2:
-        if split[0] == '-':
-            split += ['']
-        elif split[1] == '-':
-            split.insert(1, '')
-        else:
-            split.insert(1, '-')
+    inferred = infer_graph_from_sources(sources)
+    if graph == '-':
+        o = output or str(inferred.with_suffix(''))
+        return sources, None, o
 
-    # Parse components
-    if split[0] == '-':
-        g = Path(split[1])
-        r = Path(split[2]) if split[2] else g.with_suffix('')
-        return None, g, r
-    elif split[1] == '-':
-        s = Path(split[0])
-        r = Path(split[2]) if split[2] else s.with_suffix('')
-        return s, None, r
-    elif split[2] == '-':
-        s = Path(split[0])
-        g = Path(split[1]) if split[1] else s.with_suffix('.gv')
-        return s, g, None
+    if output == '-':
+        g = graph or str(inferred)
+        return sources, g, None
     else:
-        s = Path(split[0])
-        g = Path(split[1]) if split[1] else s.with_suffix('.gv')
-        r = Path(split[2]) if split[2] else g.with_suffix('')
-        return s, g, r
+        g = Path(graph) if graph else inferred
+        o = output or str(g.with_suffix(''))
+        return sources, str(g), o
