@@ -2,6 +2,7 @@ import ast
 
 from typing import List, Set, Iterable, Tuple, Optional
 from .base import Visitor, Name, Scope, Line
+from .._io import Source
 
 
 def multi_union(sets: Iterable[Set]) -> Set:
@@ -65,9 +66,8 @@ def assign_target_name(node) -> Name:
         name.is_definition = False
         return name
     else:
-        raise ValueError(
-            f'Parsing error on line {node.lineno}: unknown assignment type!'
-        )
+        deps = collect_names(node)
+        return Name(None, deps=deps, is_definition=False)
 
 
 def flatten_assign_targets(targets: List[ast.AST]) -> List[ast.AST]:
@@ -118,8 +118,28 @@ class AnnAssignVisitor(AssignVisitor):
 
 class ImportVisitor(Visitor):
     def parse_names(self) -> List[Name]:
-        names = [n.name if n.asname is None else n.asname for n in self.node.names]
-        return [Name(n, deps=set(), is_definition=True) for n in names]
+        names = []
+        for n in self.node.names:
+            source_parts = []
+            if isinstance(self.node, ast.ImportFrom):
+                source_parts += [''] * self.node.level
+                if self.node.module:
+                    source_parts += [self.node.module]
+            if n.asname:
+                name = n.asname
+                source_parts += [n.name]
+            else:
+                name = n.name.split('.')[0]
+                source_parts += [name]
+            names.append(
+                Name(
+                    name,
+                    deps=set(),
+                    is_definition=True,
+                    source='.'.join(source_parts),
+                )
+            )
+        return names
 
 
 class TryVisitor(Visitor):
@@ -380,7 +400,7 @@ def parse_scoped(visitor: Visitor, scope: Scope) -> Tuple[List[Name], Optional[s
 
         child.update_scope(scope)
 
-        assigns = {n.name for n in c_names}
+        assigns = {n.name for n in c_names if n.name is not None}
         uses = multi_union(n.deps for n in c_names)
         scope.assigned.update(assigns - scope.used)
         scope.used.update(uses)
@@ -390,7 +410,8 @@ def parse_scoped(visitor: Visitor, scope: Scope) -> Tuple[List[Name], Optional[s
 
 def parse_no_scope(visitor: Visitor) -> List[Line]:
     """Fully parse nodes as in outermost scope."""
-    self_line = Line(visitor.node, visitor.parse_names(), docstring=None)
+    names = [n for n in visitor.parse_names() if n.name is not None]
+    self_line = Line(visitor.node, names, docstring=None)
     lines = []
     previous = visitor
     for child in visitor.children():
@@ -408,6 +429,7 @@ def parse_no_scope(visitor: Visitor) -> List[Line]:
         if child.breaks_scope:
             scope = child.create_scope()
             c_names, doc = parse_scoped(child, scope)
+            c_names = [n for n in c_names if n.name is not None]
             merged = Scope()
             child.merge_scopes(merged, scope)
             deps = merged.inner_globaled | merged.inner_potential
@@ -429,9 +451,9 @@ def parse_no_scope(visitor: Visitor) -> List[Line]:
     return [line for line in lines if line.names or line.docstring]
 
 
-def parse_lines(source: str) -> List[Line]:
+def parse_lines(source: Source) -> List[Line]:
     """Parse name definitions and references on lines from source."""
-    tree = ast.parse(source)
+    tree = ast.parse(source.content)
     lines = parse_no_scope(cast(tree))
     defined_names = {n.name for line in lines for n in line.names}
 
